// polycube:  A polyomino and polycube puzzle solver.
// Copyright 2011-2018 Matthew T. Busche.
//
// This program is free software:  you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along
// with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef UTIL_HPP
#define UTIL_HPP

#include <string>
#include <iostream>

template <class InputIterator>
inline int countAll(InputIterator first, InputIterator last)
{
    int c = 0;
    for( ; first != last; ++first)
        ++c;
    return c;
}

inline void trim(std::string& s)
{
    size_t start = s.find_first_not_of(" \t");
    if(start == std::string::npos)
        s = "";
    else
        s = s.substr(start, s.find_last_not_of(" \t") + 1);
}

// Only some versions of GNU compilers can compile this __uint128_t guff
// Leaving out all reference to it unless GRIDBITFIELD_SIZE preprocessor
// definition is set to 128
//
#if GRIDBITFIELD_SIZE == 128

inline std::ostream& operator<<(std::ostream& os, __uint128_t value)
{
    char buffer[ 128 ];
    buffer[127] = 0;
    char* d = buffer + 127;
    do
    {
        --d;
        *d = "0123456789"[ value % 10 ];
        value /= 10;
    } while ( value != 0 );

    return os << d;
}

#endif

template<class T>
class Bits
{
    public:
        T t;
        Bits(const T& v) : t(v) {}
};

template<class T>
inline std::ostream& operator <<(std::ostream& os, const Bits<T>& bits)
{
    for(int shift = (sizeof(T)*8-1); shift >= 0; --shift)
    {
        T bit = 1;
        bit <<= shift;
        os << ((bits.t&bit) ? "1" : "0");
    }
    return os;
}

#endif
